@@ -34,11 +34,11 @@
 #include <pthread.h>
 
 #include "daemon/cfg.h"
-#include "daemon/cmdhandler.h"
+#include "daemon/enforcercommands.h"
 #include "clientpipe.h"
+#include "cmdhandler.h"
+#include "locks.h"
 #include "daemon/engine.h"
-#include "daemon/signal.h"
-#include "daemon/worker.h"
 #include "scheduler/schedule.h"
 #include "scheduler/task.h"
 #include "file.h"
@@ -51,6 +51,7 @@
 #include "db/database_version.h"
 #include "hsmkey/hsm_key_factory.h"
 #include "libhsm.h"
+#include "locks.h"
 
 #include <errno.h>
 #include <libxml/parser.h>
@@ -68,6 +69,8 @@
 
 static const char* engine_str = "engine";
 
+static engine_type* engine = NULL;
+
 /**
  * Create engine.
  *
@@ -75,12 +78,10 @@ static const char* engine_str = "engine";
 engine_type*
 engine_alloc(void)
 {
-    engine_type* engine;
     engine = (engine_type*) malloc(sizeof(engine_type));
     if (!engine) return NULL;
 
     pthread_mutex_init(&engine->signal_lock, NULL);
-    pthread_mutex_init(&engine->enforce_lock, NULL);
     pthread_cond_init(&engine->signal_cond, NULL);
 
     engine->dbcfg_list = NULL;
@@ -96,7 +97,6 @@ void
 engine_dealloc(engine_type* engine)
 {
     schedule_cleanup(engine->taskq);
-    pthread_mutex_destroy(&engine->enforce_lock);
     pthread_mutex_destroy(&engine->signal_lock);
     pthread_cond_destroy(&engine->signal_cond);
     if (engine->dbcfg_list) {
@@ -106,33 +106,12 @@ engine_dealloc(engine_type* engine)
     free(engine);
 }
 
-/**
- * Start command handler.
- *
- */
-static void*
-cmdhandler_thread_start(void* arg)
-{
-    int err;
-    sigset_t sigset;
-    cmdhandler_type* cmd = (cmdhandler_type*) arg;
-
-    sigfillset(&sigset);
-    if((err=pthread_sigmask(SIG_SETMASK, &sigset, NULL)))
-        ods_fatal_exit("[%s] pthread_sigmask: %s", engine_str, strerror(err));
-
-    cmdhandler_start(cmd);
-    return NULL;
-}
-
 static void
 engine_start_cmdhandler(engine_type* engine)
 {
     ods_log_assert(engine);
     ods_log_debug("[%s] start command handler", engine_str);
-    engine->cmdhandler->engine = engine;
-    pthread_create(&engine->cmdhandler->thread_id, NULL,
-        cmdhandler_thread_start, engine->cmdhandler);
+    janitor_thread_create(&engine->cmdhandler->thread_id, workerthreadclass, (janitor_runfn_t)cmdhandler_start, engine->cmdhandler);
 }
 
 /**
@@ -179,37 +158,18 @@ engine_privdrop(engine_type* engine)
 static void
 engine_create_workers(engine_type* engine)
 {
-    size_t i = 0;
+    char* name;
+    int i = 0;
     ods_log_assert(engine);
     ods_log_assert(engine->config);
     engine->workers = (worker_type**) malloc(
         (size_t)engine->config->num_worker_threads * sizeof(worker_type*));
     for (i=0; i < (size_t) engine->config->num_worker_threads; i++) {
-        engine->workers[i] = worker_create(i);
+        asprintf(&name, "worker[%d]", i+1);
+        engine->workers[i] = worker_create(name, engine->taskq);
     }
 }
 
-static void*
-worker_thread_start(void* arg)
-{
-    int err;
-    sigset_t sigset;
-    worker_type* worker = (worker_type*) arg;
-
-    sigfillset(&sigset);
-    if((err=pthread_sigmask(SIG_SETMASK, &sigset, NULL)))
-        ods_fatal_exit("[%s] pthread_sigmask: %s", engine_str, strerror(err));
-
-    worker->dbconn = get_database_connection(worker->engine->dbcfg_list);
-    if (!worker->dbconn) {
-        ods_log_crit("Failed to start worker, could not connect to database");
-        return NULL;
-    }
-    worker_start(worker);
-    db_connection_free(worker->dbconn);
-    return NULL;
-}
-
 void
 engine_start_workers(engine_type* engine)
 {
@@ -220,9 +180,12 @@ engine_start_workers(engine_type* engine)
     ods_log_debug("[%s] start workers", engine_str);
     for (i=0; i < (size_t) engine->config->num_worker_threads; i++) {
         engine->workers[i]->need_to_exit = 0;
-        engine->workers[i]->engine = (struct engine_struct*) engine;
-        pthread_create(&engine->workers[i]->thread_id, NULL,
-            worker_thread_start, engine->workers[i]);
+        engine->workers[i]->context = get_database_connection(engine);
+        if (!engine->workers[i]->context) {
+            ods_log_crit("Failed to start worker, could not connect to database");
+        } else {
+            janitor_thread_create(&engine->workers[i]->thread_id, workerthreadclass, (janitor_runfn_t)worker_start, engine->workers[i]);
+        }
     }
 }
 
@@ -242,8 +205,8 @@ engine_stop_workers(engine_type* engine)
     /* head count */
     for (i=0; i < engine->config->num_worker_threads; i++) {
         ods_log_debug("[%s] join worker %i", engine_str, i+1);
-        (void)pthread_join(engine->workers[i]->thread_id, NULL);
-        engine->workers[i]->engine = NULL;
+        janitor_thread_join(engine->workers[i]->thread_id);
+        db_connection_free(engine->workers[i]->context);
     }
 }
 
@@ -260,12 +223,12 @@ engine_wakeup_workers(engine_type* engine)
 }
 
 db_connection_t*
-get_database_connection(db_configuration_list_t* dbcfg_list)
+get_database_connection(engine_type* engine)
 {
     db_connection_t* dbconn;
 
     if (!(dbconn = db_connection_new())
-        || db_connection_set_configuration_list(dbconn, dbcfg_list)
+        || db_connection_set_configuration_list(dbconn, engine->dbcfg_list)
         || db_connection_setup(dbconn)
         || db_connection_connect(dbconn))
     {
@@ -282,12 +245,12 @@ get_database_connection(db_configuration_list_t* dbcfg_list)
  * \return 0 on success, 1 on failure.
  */
 static int
-probe_database(db_configuration_list_t* dbcfg_list)
+probe_database(engine_type* engine)
 {
     db_connection_t *conn;
     int version;
 
-    conn = get_database_connection(dbcfg_list);
+    conn = get_database_connection(engine);
     if (!conn) return 1;
     version = database_version_get_version(conn);
     db_connection_free(conn);
@@ -435,19 +398,46 @@ desetup_database(engine_type* engine)
     engine->dbcfg_list = NULL;
 }
 
+static void *
+signal_handler(sig_atomic_t sig)
+{
+    switch (sig) {
+        case SIGHUP:
+            if (engine) {
+                engine->need_to_reload = 1;
+                pthread_mutex_lock(&engine->signal_lock);
+                pthread_cond_signal(&engine->signal_cond);
+                pthread_mutex_unlock(&engine->signal_lock);
+            }
+            break;
+        case SIGINT:
+        case SIGTERM:
+            if (engine) {
+                engine->need_to_exit = 1;
+                pthread_mutex_lock(&engine->signal_lock);
+                pthread_cond_signal(&engine->signal_cond);
+                pthread_mutex_unlock(&engine->signal_lock);
+            }
+            break;
+        default:
+            break;
+    }
+    return NULL;
+}
+
 /**
  * Set up engine and return the setup status.
  *
  */
 ods_status
-engine_setup(engine_type* engine)
+engine_setup()
 {
-    int fd;
+    int fd, error;
+    int pipefd[2];
+    char buff = '\0';
 
     ods_log_debug("[%s] enforcer setup", engine_str);
 
-    ods_log_init("ods-enforcerd", engine->config->use_syslog, engine->config->log_filename, engine->config->verbosity);
-
     engine->pid = getpid(); /* We need to do this again after fork() */
 
     if (!util_pidfile_avail(engine->config->pid_filename)) {
@@ -457,20 +447,25 @@ engine_setup(engine_type* engine)
     /* setup database configuration */
     if (setup_database(engine)) return ODS_STATUS_DB_ERR;
     /* Probe the database, can we connect to it? */
-    if (probe_database(engine->dbcfg_list)) {
+    if (probe_database(engine)) {
         ods_log_crit("Could not connect to database or database not set"
             " up properly.");
         return ODS_STATUS_DB_ERR;
     }
 
     /* create command handler (before chowning socket file) */
-    engine->cmdhandler = cmdhandler_create(engine->config->clisock_filename);
+    engine->cmdhandler = cmdhandler_create(engine->config->clisock_filename, enforcercommands, engine, (void*(*)(void*)) (void(*)(void*))&get_database_connection, (void(*)(void*))&db_connection_free);
     if (!engine->cmdhandler) {
         ods_log_error("[%s] create command handler to %s failed",
             engine_str, engine->config->clisock_filename);
         return ODS_STATUS_CMDHANDLER_ERR;
     }
 
+    if(pipe(pipefd)) {
+        ods_log_error("[%s] unable to pipe: %s", engine_str, strerror(errno));
+        return ODS_STATUS_PIPE_ERR;
+    }
+
     if (!engine->init_setup_done) {
         /* privdrop */
         engine->uid = privuid(engine->config->username);
@@ -508,14 +503,31 @@ engine_setup(engine_type* engine)
                         (void)dup2(fd, STDERR_FILENO);
                         if (fd > 2) (void)close(fd);
                     }
-                    engine->daemonize = 0; /* don't fork again on reload */
+                    close(pipefd[0]);
                     break;
                 default: /* parent */
+                    close(pipefd[1]);
+                    /* Print messages the child may have send us. */
+                    while (read(pipefd[0], &buff, 1) != -1) {
+                        if (buff <= 1) break;
+                        printf("%c", buff);
+                    }
+                    close(pipefd[0]);
+                    if (buff != '\1') {
+                        ods_log_error("[%s] fail to start enforcerd completely", engine_str);
+                        exit(1);
+                    }
+                    ods_log_debug("[%s] enforcerd started successfully", engine_str);
                     exit(0);
             }
             if (setsid() == -1) {
                 ods_log_error("[%s] unable to setsid daemon (%s)",
                     engine_str, strerror(errno));
+                const char *err = "unable to setsid daemon: ";
+                ods_writen(pipefd[1], err, strlen(err));
+                ods_writeln(pipefd[1], strerror(errno));
+                write(pipefd[1], "\0", 1);
+                close(pipefd[1]);
                 return ODS_STATUS_SETSID_ERR;
             }
         }
@@ -529,16 +541,40 @@ engine_setup(engine_type* engine)
     /* create workers */
     engine_create_workers(engine);
 
-    /* start command handler */
-    engine->cmdhandler_done = 0;
-
     /* write pidfile */
     if (util_write_pidfile(engine->config->pid_filename, engine->pid) == -1) {
         hsm_close();
         ods_log_error("[%s] unable to write pid file", engine_str);
+        if (engine->daemonize) {
+            ods_writeln(pipefd[1], "unable to write pid file");
+            write(pipefd[1], "\0", 1);
+            close(pipefd[1]);
+        }
         return ODS_STATUS_WRITE_PIDFILE_ERR;
     }
+    ods_log_info("[%s] enforcer started", engine_str);
+    error = hsm_open2(engine->config->repositories, hsm_prompt_pin);
+    if (error != HSM_OK) {
+        char* errorstr =  hsm_get_error(NULL);
+        if (!errorstr)
+            (void)asprintf(&errorstr, "error opening libhsm (errno %i)", error);
+        if (errorstr)
+            ods_log_error("[%s] %s", engine_str, errorstr);
+        if (engine->daemonize) {
+            if (errorstr) ods_writeln(pipefd[1], errorstr);
+            write(pipefd[1], "\0", 1);
+            close(pipefd[1]);
+        }
+        free(errorstr);
+        return ODS_STATUS_HSM_ERR;
+    }
+    engine->need_to_reload = 0;
+    engine_start_cmdhandler(engine);
 
+    write(pipefd[1], "\1", 1);
+    close(pipefd[1]);
+    if (!engine->daemonize) close(pipefd[0]);
+    engine->daemonize = 0; /* don't fork again on reload */
     return ODS_STATUS_OK;
 }
 
@@ -566,7 +602,7 @@ engine_teardown(engine_type* engine)
         }
         free(engine->workers);
         engine->workers = NULL;
-    }
+    } 
     if (engine->cmdhandler) {
         cmdhandler_cleanup(engine->cmdhandler);
         engine->cmdhandler = NULL;
@@ -582,7 +618,6 @@ engine_init(engine_type* engine, int daemonize)
     engine->config = NULL;
     engine->workers = NULL;
     engine->cmdhandler = NULL;
-    engine->cmdhandler_done = 1;
     engine->init_setup_done = 0;
     engine->pid = getpid(); /* We need to do this again after fork() */
     engine->uid = -1;
@@ -591,7 +626,6 @@ engine_init(engine_type* engine, int daemonize)
     engine->need_to_reload = 0;
     engine->daemonize = daemonize;
     /* catch signals */
-    signal_set_engine(engine);
     action.sa_handler = (void (*)(int))signal_handler;
     sigfillset(&action.sa_mask);
     action.sa_flags = 0;
@@ -610,28 +644,12 @@ engine_run(engine_type* engine, start_cb_t start, int single_run)
 {
     int error;
     ods_log_assert(engine);
-    ods_log_info("[%s] enforcer started", engine_str);
-    
-    error = hsm_open2(engine->config->repositories, hsm_prompt_pin);
-    if (error != HSM_OK) {
-        char* errorstr =  hsm_get_error(NULL);
-        if (errorstr != NULL) {
-            ods_log_error("[%s] %s", engine_str, errorstr);
-            free(errorstr);
-        } else {
-            ods_log_crit("[%s] error opening libhsm (errno %i)", engine_str,
-                error);
-        }
-        return 1;
-    }
-    
-    engine->need_to_reload = 0;
-    engine_start_cmdhandler(engine);
+
     engine_start_workers(engine);
 
     /* call the external start callback function */
     start(engine);
-    
+
     while (!engine->need_to_exit && !engine->need_to_reload) {
         if (single_run) {
             engine->need_to_exit = 1;
@@ -654,7 +672,7 @@ engine_run(engine_type* engine, start_cb_t start, int single_run)
     }
     ods_log_debug("[%s] enforcer halted", engine_str);
     engine_stop_workers(engine);
-    cmdhandler_stop(engine);
+    cmdhandler_stop(engine->cmdhandler);
     schedule_purge(engine->taskq); /* Remove old tasks in queue */
     hsm_close();
     return 0;
