@@ -28,6 +28,6 @@
 #ifndef _ENFORCER_REPOSITORYLIST_CMD_H_
 #define _ENFORCER_REPOSITORYLIST_CMD_H_
 
-struct cmd_func_block* repositorylist_funcblock(void);
+struct cmd_func_block repositorylist_funcblock;
 
 #endif
