@@ -27,18 +27,21 @@
  *
  */
 
+#include <getopt.h>
 #include "config.h"
 
-#include "daemon/cmdhandler.h"
+#include "cmdhandler.h"
+#include "daemon/enforcercommands.h"
 #include "daemon/engine.h"
 #include "file.h"
+#include "duration.h"
 #include "log.h"
 #include "str.h"
 #include "clientpipe.h"
 
 #include "db/key_state.h"
 #include "db/hsm_key.h"
-#include "db/zone.h"
+#include "db/zone_db.h"
 
 #include "keystate/keystate_list_cmd.h"
 
@@ -51,9 +54,9 @@ static const char *module_str = "keystate_list_task";
 #define UNR KEY_STATE_STATE_UNRETENTIVE
 #define NAV KEY_STATE_STATE_NA
 
-enum {KS_GEN = 0, KS_PUB, KS_RDY, KS_ACT, KS_RET, KS_DEA, KS_UNK, KS_MIX};
+enum {KS_GEN = 0, KS_PUB, KS_RDY, KS_ACT, KS_RET, KS_UNK, KS_MIX};
 const char* statenames[] = {"generate", "publish", "ready",
-		"active", "retire", "dead", "unknown", "mixed"};
+		"active", "retire", "unknown", "mixed"};
 
 /** Map 2.0 states to 1.x states
  * @param p: state of RR higher in the chain (e.g. DS)
@@ -73,12 +76,14 @@ keystate(int p, int c, int introducing, key_data_ds_at_parent_t dsstate)
 		if (p == HID && c == HID) return KS_GEN;
 		if (p == HID || c == HID) return KS_PUB;
 		if (p == OMN || c == OMN) return KS_RDY;
+		if (p == RUM || c == RUM) return KS_RDY;
 		return KS_UNK;
 	} else {
 		/* retire conforms better to 1.4 terminology than dead. */
 		if (p == HID && c == HID) return KS_RET; /* dead */
 		if (p == UNR || c == UNR) return KS_RET;
 		if (p == OMN || c == OMN) return KS_RDY;
+		if (p == RUM || c == RUM) return KS_RDY;
 		return KS_RET;
 	}
 }
@@ -128,7 +133,7 @@ map_keystate(key_data_t *key)
  * @param key: key to evaluate
  * @return: human readable transition time/event */
 static char*
-map_keytime(const zone_t *zone, const key_data_t *key)
+map_keytime(const zone_db_t *zone, const key_data_t *key)
 {
 	char ct[26];
 	struct tm srtm;
@@ -146,10 +151,12 @@ map_keytime(const zone_t *zone, const key_data_t *key)
 		default:
 			break;
 	}
-	if (zone_next_change(zone) < 0)
+	if (zone_db_next_change(zone) < 0)
 		return strdup("-");
+	else if (zone_db_next_change(zone) < time_now())
+		return strdup("now");
 
-	t = (time_t)zone_next_change(zone);
+	t = (time_t)zone_db_next_change(zone);
 	localtime_r(&t, &srtm);
 	strftime(ct, 26, "%Y-%m-%d %H:%M:%S", &srtm);
 	return strdup(ct);
@@ -157,16 +164,15 @@ map_keytime(const zone_t *zone, const key_data_t *key)
 
 static int
 perform_keystate_list(int sockfd, db_connection_t *dbconn,
-        const char* filterZone, char** filterKeytype, char** filterKeystate,
-        void (printheader)(int sockfd),
-        void (printkey)(int sockfd, zone_t* zone, key_data_t* key, char*tchange, hsm_key_t* hsmKey)) {
+    const char* zonename, const char* keytype, const char* keystate,
+    void (printheader)(int sockfd),
+    void (printkey)(int sockfd, zone_db_t* zone, key_data_t* key, char* tchange, hsm_key_t* hsmKey)) {
     key_data_list_t* key_list;
     key_data_t* key;
-    zone_t *zone = NULL;
+    zone_db_t *zone = NULL;
     char* tchange;
     hsm_key_t *hsmkey;
     int cmp;
-    int i, skipPrintKey;
 
     if (!(key_list = key_data_list_new_get(dbconn))) {
         client_printf_err(sockfd, "Unable to get list of keys, memory "
@@ -181,9 +187,9 @@ perform_keystate_list(int sockfd, db_connection_t *dbconn,
     while ((key = key_data_list_get_next(key_list))) {
 		/* only refetches zone if different from previous */
         if (zone
-                && (db_value_cmp(zone_id(zone), key_data_zone_id(key), &cmp)
+                && (db_value_cmp(zone_db_id(zone), key_data_zone_id(key), &cmp)
                 || cmp)) {
-            zone_free(zone);
+            zone_db_free(zone);
             zone = NULL;
         }
         if (!zone) {
@@ -192,29 +198,13 @@ perform_keystate_list(int sockfd, db_connection_t *dbconn,
         hsmkey = key_data_get_hsm_key(key);
         key_data_cache_key_states(key);
         tchange = map_keytime(zone, key); /* allocs */
-        skipPrintKey = 0;
-        if(printkey == NULL)
-            skipPrintKey = 1;
-        if(filterZone != NULL && strcmp(zone_name(zone), filterZone))
-            skipPrintKey = 1;
-        for(i=0; filterKeytype && filterKeytype[i]; i++)
-            if(!strcasecmp(filterKeytype[i],key_data_role_text(key)))
-                break;
-        if(filterKeytype && filterKeytype[i] == NULL)
-            skipPrintKey = 1;
-        for(i=0; filterKeystate && filterKeystate[i]; i++)
-            if(!strcasecmp(filterKeystate[i],map_keystate(key)))
-                break;
-        if(filterKeystate && filterKeystate[i] == NULL)
-            skipPrintKey = 1;
-        if (!skipPrintKey) {
+        if ((printkey != NULL) && (!zonename || !strcmp(zone_db_name(zone), zonename)) && (!keytype || !strcasecmp(keytype,key_data_role_text(key))) && (!keystate || !strcasecmp(keystate, map_keystate(key))))
             (*printkey)(sockfd, zone, key, tchange, hsmkey);
-        }
         free(tchange);
         hsm_key_free(hsmkey);
         key_data_free(key);
     }
-    zone_free(zone);
+    zone_db_free(zone);
     key_data_list_free(key_list);
     return 0;
 }
@@ -247,12 +237,6 @@ help(int sockfd)
 		"all		print keys in all states (including generate) \n\n");
 }
 
-static int
-handles(const char *cmd, ssize_t n)
-{
-	return ods_check_command(cmd, n, key_list_funcblock()->cmdname)?1:0;
-}
-
 static void
 printcompatheader(int sockfd) {
     client_printf(sockfd, "Keys:\n");
@@ -261,11 +245,11 @@ printcompatheader(int sockfd) {
 }
 
 static void
-printcompatkey(int sockfd, zone_t* zone, key_data_t* key, char*tchange, hsm_key_t* hsmkey) {
+printcompatkey(int sockfd, zone_db_t* zone, key_data_t* key, char* tchange, hsm_key_t* hsmkey) {
     (void)hsmkey;
     client_printf(sockfd,
             "%-31s %-8s %-9s %s\n",
-            zone_name(zone),
+            zone_db_name(zone),
             key_data_role_text(key),
             map_keystate(key),
             tchange);
@@ -280,11 +264,11 @@ printverboseheader(int sockfd) {
 }
 
 static void
-printverbosekey(int sockfd, zone_t* zone, key_data_t* key, char* tchange, hsm_key_t* hsmkey) {
+printverbosekey(int sockfd, zone_db_t* zone, key_data_t* key, char* tchange, hsm_key_t* hsmkey) {
     (void)tchange;
     client_printf(sockfd,
             "%-31s %-8s %-9s %-24s %-5d %-10d %-32s %-11s %d\n",
-            zone_name(zone),
+            zone_db_name(zone),
             key_data_role_text(key),
             map_keystate(key),
             tchange,
@@ -296,10 +280,10 @@ printverbosekey(int sockfd, zone_t* zone, key_data_t* key, char* tchange, hsm_ke
 }
 
 static void
-printverboseparsablekey(int sockfd, zone_t* zone, key_data_t* key, char* tchange, hsm_key_t* hsmkey) {
+printverboseparsablekey(int sockfd, zone_db_t* zone, key_data_t* key, char* tchange, hsm_key_t* hsmkey) {
     client_printf(sockfd,
             "%s;%s;%s;%s;%d;%d;%s;%s;%d\n",
-            zone_name(zone),
+            zone_db_name(zone),
             key_data_role_text(key),
             map_keystate(key),
             tchange,
@@ -319,11 +303,11 @@ printdebugheader(int sockfd) {
 }
 
 static void
-printdebugkey(int sockfd, zone_t* zone, key_data_t* key, char* tchange, hsm_key_t* hsmkey) {
+printdebugkey(int sockfd, zone_db_t* zone, key_data_t* key, char* tchange, hsm_key_t* hsmkey) {
     (void)tchange;
     client_printf(sockfd,
             "%-31s %-13s %-12s %-12s %-12s %-12s %d %4d    %s\n",
-            zone_name(zone),
+            zone_db_name(zone),
             key_data_role_text(key),
             key_state_state_text(key_data_cached_ds(key)),
             key_state_state_text(key_data_cached_dnskey(key)),
@@ -335,11 +319,11 @@ printdebugkey(int sockfd, zone_t* zone, key_data_t* key, char* tchange, hsm_key_
 }
 
 static void
-printdebugparsablekey(int sockfd, zone_t* zone, key_data_t* key, char* tchange, hsm_key_t* hsmkey) {
+printdebugparsablekey(int sockfd, zone_db_t* zone, key_data_t* key, char* tchange, hsm_key_t* hsmkey) {
     (void)tchange;
     client_printf(sockfd,
             "%s;%s;%s;%s;%s;%s;%d;%d;%s\n",
-            zone_name(zone),
+            zone_db_name(zone),
             key_data_role_text(key),
             key_state_state_text(key_data_cached_ds(key)),
             key_state_state_text(key_data_cached_dnskey(key)),
@@ -350,156 +334,103 @@ printdebugparsablekey(int sockfd, zone_t* zone, key_data_t* key, char* tchange,
             hsm_key_locator(hsmkey));
 }
 
-static char **
-tokenizeparam(char *argument)
-{
-    char** tokenized;
-    char** newtokenized;
-    int argCount;
-    char* argString;
-    char* argSavePtr = NULL;
-    int argSize = 8;
-
-    if ((argString = strtok_r(argument, ",", &argSavePtr)) != NULL) {
-        if ((tokenized = malloc(sizeof (char*)*argSize)) == NULL) {
-            return NULL;
-        }
-        argCount = 0;
-        do {
-            if (strcmp(argString, "")) {
-                tokenized[argCount] = argString;
-                ++argCount;
-                if (argCount == argSize) {
-                    argSize *= 2;
-                    if ((newtokenized = realloc(tokenized, sizeof (char*)*argSize)) == NULL) {
-                        free(tokenized);
-                        return NULL;
-                    }
-                    tokenized = newtokenized;
-                }
-            }
-        } while (strtok_r(NULL, ",", &argSavePtr) != NULL);
-        tokenized[argCount] = NULL;
-    } else {
-        if ((tokenized = malloc(sizeof (char*)*2)) == NULL) {
-            return NULL;
-        }
-        tokenized[0] = argument;
-        tokenized[1] = NULL;
-    }
-    return tokenized;
-}
-
 static int
-run(int sockfd, engine_type* engine, const char *cmd, ssize_t n,
-	db_connection_t *dbconn) {
+run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
+{
     char buf[ODS_SE_MAXLINE];
-#define NARGV 12
+    #define NARGV 12
     const char *argv[NARGV];
     int success, argIndex;
-    int argc, bVerbose, bDebug, bParsable, bAll;
-    char* keytypeParam;
-    char* keystateParam;
-    const char* filterZone; /* NULL if no filtering on zone, otherwise zone to match */
-    char** filterKeytype; /* NULL if no filtering on key type, NULL terminated list of key types to filter */
-    char** filterKeystate; /* NULL if no filtering on key state, NULL terminated list of key states to filter */
-    (void) engine;
+    int argc = 0, bVerbose = 0, bDebug = 0, bParsable = 0, bAll = 0;
+    int long_index = 0, opt = 0;
+    const char* keytype = NULL;
+    const char* keystate = NULL;
+    const char* zonename = NULL;
+    db_connection_t* dbconn = getconnectioncontext(context);
 
-    ods_log_debug("[%s] %s command", module_str, key_list_funcblock()->cmdname);
+    static struct option long_options[] = {
+        {"verbose", no_argument, 0, 'v'},
+        {"debug", no_argument, 0, 'd'},
+        {"parsable", no_argument, 0, 'p'},
+        {"zone", required_argument, 0, 'z'},
+        {"keytype", required_argument, 0, 't'},
+        {"keystate", required_argument, 0, 'e'},
+        {"all", no_argument, 0, 'a'},
+        {0, 0, 0, 0}
+    };
+
+    ods_log_debug("[%s] %s command", module_str, key_list_funcblock.cmdname);
 
-    cmd = ods_check_command(cmd, n, key_list_funcblock()->cmdname);
     /* Use buf as an intermediate buffer for the command. */
     strncpy(buf, cmd, sizeof (buf));
     buf[sizeof (buf) - 1] = '\0';
 
     /* separate the arguments */
     argc = ods_str_explode(buf, NARGV, argv);
-    if (argc > NARGV) {
-        ods_log_warning("[%s] too many arguments for %s command",
-                module_str, key_list_funcblock()->cmdname);
-        client_printf(sockfd, "too many arguments\n");
+    if (argc == -1) {
+        ods_log_error("[%s] too many arguments for %s command",
+                module_str, key_list_funcblock.cmdname);
+        client_printf_err(sockfd, "too many arguments\n");
         return -1;
     }
-
-    bVerbose = ods_find_arg(&argc, argv, "verbose", "v") != -1;
-    bDebug = ods_find_arg(&argc, argv, "debug", "d") != -1;
-    bParsable = ods_find_arg(&argc, argv, "parsable", "p") != -1;
-    if ((argIndex = ods_find_arg_and_param(&argc, argv, "zone", "z", &filterZone)) == -1) {
-        filterZone = NULL;
-    }
-    if (ods_find_arg_and_param(&argc, argv, "keytype", "k", (const char **)&keytypeParam) == -1) {
-        keytypeParam = NULL;
-    }
-    if (ods_find_arg_and_param(&argc, argv, "keystate", "e", (const char **)&keystateParam) == -1) {
-        keystateParam = NULL;
+    optind = 0;
+    while ((opt = getopt_long(argc, (char* const*)argv, "vdpz:t:e:a", long_options, &long_index) ) != -1) {
+        switch (opt) {
+            case 'v':
+                bVerbose = 1;
+                break;
+            case 'd':
+                bDebug = 1;
+                break;
+            case 'p':
+                bParsable = 1;
+                break;
+            case 'z':
+                zonename = optarg;
+                break;
+            case 't':
+                keytype = optarg;
+                break;
+            case 'e':
+                keystate = optarg;
+                break;
+            case 'a':
+                bAll = 1;
+                break;
+            default:
+                client_printf_err(sockfd, "unknown arguments\n");
+                ods_log_error("[%s] unknown arguments for %s command",
+                              module_str, key_list_funcblock.cmdname);
+                return -1;
+        }
     }
 
-    bAll = (ods_find_arg(&argc, argv, "all", "a") != -1);
-
-    if (keystateParam != NULL && bAll) {
+    if (keystate != NULL && bAll) {
         client_printf(sockfd, "Error: --keystate and --all option cannot be given together\n");
         return -1;
     }
 
-    if (argc) {
-        ods_log_warning("[%s] unknown arguments for %s command", module_str, key_list_funcblock()->cmdname);
-        client_printf(sockfd, "unknown arguments\n");
-        return -1;
-    }
-
-    if (keytypeParam)
-        filterKeytype = tokenizeparam(keytypeParam);
-    else
-        filterKeytype = NULL;
-    if (keystateParam) {
-        filterKeystate = tokenizeparam(keystateParam);
-    } else
-        filterKeystate = NULL;
-    if (bAll) {
-        if (filterKeystate != NULL) {
-            free(filterKeystate);
-        }
-        filterKeystate = NULL;
-    } else if(filterKeystate == NULL) {
-        if ((filterKeystate = malloc(sizeof (char*) * 6))) {
-            filterKeystate[0] = (char *)"publish";
-            filterKeystate[1] = (char *)"ready";
-            filterKeystate[2] = (char *)"active";
-            filterKeystate[3] = (char *)"retire";
-            filterKeystate[4] = (char *)"mixed";
-            filterKeystate[5] = NULL;
-        } /* else emit error */
-    }
-
     if (bDebug) {
         if (bParsable) {
-            success = perform_keystate_list(sockfd, dbconn, filterZone, filterKeytype, filterKeystate, NULL, &printdebugparsablekey);
+            success = perform_keystate_list(sockfd, dbconn, zonename, keytype, keystate, NULL, &printdebugparsablekey);
         } else {
-            success = perform_keystate_list(sockfd, dbconn, filterZone, filterKeytype, filterKeystate, &printdebugheader, &printdebugkey);
+            success = perform_keystate_list(sockfd, dbconn, zonename, keytype, keystate, &printdebugheader, &printdebugkey);
         }
     } else if (bVerbose) {
         if (bParsable) {
-            success = perform_keystate_list(sockfd, dbconn, filterZone, filterKeytype, filterKeystate, NULL, &printverboseparsablekey);
+            success = perform_keystate_list(sockfd, dbconn, zonename, keytype, keystate, NULL, &printverboseparsablekey);
         } else {
-            success = perform_keystate_list(sockfd, dbconn, filterZone, filterKeytype, filterKeystate, &printverboseheader, &printverbosekey);
+            success = perform_keystate_list(sockfd, dbconn, zonename, keytype, keystate, &printverboseheader, &printverbosekey);
         }
     } else {
-        success = perform_keystate_list(sockfd, dbconn, filterZone, filterKeytype, filterKeystate, &printcompatheader, &printcompatkey);
+        if (bParsable)
+            client_printf_err(sockfd, "-p option only available in combination with -v and -d.\n");
+        success = perform_keystate_list(sockfd, dbconn, zonename, keytype, keystate, &printcompatheader, &printcompatkey);
     }
 
-    if (filterKeytype)
-        free(filterKeytype);
-    if (filterKeystate)
-        free(filterKeystate);
     return success;
 }
 
-static struct cmd_func_block funcblock = {
-	"key list", &usage, &help, &handles, &run
+struct cmd_func_block key_list_funcblock = {
+	"key list", &usage, &help, NULL, &run
 };
-
-struct cmd_func_block*
-key_list_funcblock(void)
-{
-	return &funcblock;
-}
