@@ -216,6 +216,7 @@ parse_conf_repositories(const char* cfgfile)
     char* tokenlabel;
     char* pin;
     uint8_t use_pubkey;
+    uint8_t allowextract;
     int require_backup;
     hsm_repository_t* rlist = NULL;
     hsm_repository_t* repo  = NULL;
@@ -254,6 +255,7 @@ parse_conf_repositories(const char* cfgfile)
             tokenlabel = NULL;
             pin = NULL;
             use_pubkey = 1;
+            allowextract = 0;
             require_backup = 0;
 
             curNode = xpathObj->nodesetval->nodeTab[i]->xmlChildrenNode;
@@ -270,12 +272,14 @@ parse_conf_repositories(const char* cfgfile)
                     pin = (char *) xmlNodeGetContent(curNode);
                 if (xmlStrEqual(curNode->name, (const xmlChar *)"SkipPublicKey"))
                     use_pubkey = 0;
+                if (xmlStrEqual(curNode->name, (const xmlChar *)"AllowExtraction"))
+                    allowextract = 1;
 
                 curNode = curNode->next;
             }
             if (name && module && tokenlabel) {
                 repo = hsm_repository_new(name, module, tokenlabel, pin,
-                    use_pubkey, require_backup);
+                    use_pubkey, allowextract, require_backup);
             }
             if (!repo) {
                ods_log_error("[%s] unable to add %s repository: "
@@ -715,3 +719,23 @@ parse_conf_automatic_keygen_period(const char* cfgfile)
     }
     return period;
 }
+
+time_t
+parse_conf_rollover_notification(const char* cfgfile)
+{
+    time_t period = 0;
+    const char* str = parse_conf_string(cfgfile,
+                                        "//Configuration/Enforcer/RolloverNotification",
+                                        0);
+    if (str) {
+        if (strlen(str) > 0) {
+            duration_type* duration = duration_create_from_string(str);
+            if (duration) {
+                period = duration2time(duration);
+                duration_cleanup(duration);
+            }
+        }
+        free((void*)str);
+    }
+    return period;
+}
