@@ -24,17 +24,9 @@
  *
  */
 
-/**
- * Signing keys.
- *
- */
-
 #ifndef SIGNER_KEYS_H
 #define SIGNER_KEYS_H
 
-#include "shared/allocator.h"
-#include "shared/status.h"
-
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
@@ -42,35 +34,37 @@
 # include <unistd.h>
 #endif
 #include <ldns/ldns.h>
-#include <libhsm.h>
-#include <libhsmdns.h>
 
+typedef struct key_struct key_type;
+typedef struct keylist_struct keylist_type;
+
+#include "status.h"
+#include "libhsm.h"
+#include "libhsmdns.h"
+#include "signconf.h"
 
 /**
  * Key.
  *
  */
-typedef struct key_struct key_type;
 struct key_struct {
     ldns_rr* dnskey;
-    hsm_key_t* hsmkey;
     hsm_sign_params_t* params;
     const char* locator;
+    const char* resourcerecord;
     uint8_t algorithm;
     uint32_t flags;
     int publish;
     int ksk;
     int zsk;
-    int rfc5011;
 };
 
 /**
  * Key list.
  *
  */
-typedef struct keylist_struct keylist_type;
 struct keylist_struct {
-    void* sc;
+    signconf_type* sc;
     key_type* keys;
     size_t count;
 };
@@ -81,7 +75,7 @@ struct keylist_struct {
  * \return keylist_type* key list
  *
  */
-keylist_type* keylist_create(void* sc);
+keylist_type* keylist_create(signconf_type* sc);
 
 /**
  * Lookup a key in the key list by locator.
@@ -92,15 +86,6 @@ keylist_type* keylist_create(void* sc);
  */
 key_type* keylist_lookup_by_locator(keylist_type* kl, const char* locator);
 
-/**
- * Lookup a key in the key list by dnskey.
- * \param[in] kl key list
- * \param[in] dnskey dnskey
- * \return key_type* key if it exists, NULL otherwise
- *
- */
-key_type* keylist_lookup_by_dnskey(keylist_type* kl, ldns_rr* dnskey);
-
 /**
  * Push a key to the keylist.
  * \param[in] kl key list
@@ -110,21 +95,11 @@ key_type* keylist_lookup_by_dnskey(keylist_type* kl, ldns_rr* dnskey);
  * \param[in] publish if true, publish key as a DNSKEY
  * \param[in] ksk if true, sign DNSKEY RRset with this key
  * \param[in] zsk if true, sign all but DNSKEY RRset with this key
- * \param[in] rfc5011 if true, key will use 5011 style revocation
  * \return key_type* key
  *
  */
-key_type* keylist_push(keylist_type* kl, const char* locator,
-    uint8_t algorithm, uint32_t flags, int publish, int ksk, int zsk,
-    int rfc5011);
-
-/**
- * Print key list.
- * \param[in] fd file descriptor
- * \param[in] kl key list to print
- *
- */
-void keylist_print(FILE* fd, keylist_type* kl);
+key_type* keylist_push(keylist_type* kl, const char* locator, const char* resourcerecord,
+    uint8_t algorithm, uint32_t flags, int publish, int ksk, int zsk);
 
 /**
  * Log key list.
