@@ -47,19 +47,10 @@ static const char* edns_str = "edns";
  *
  */
 edns_rr_type*
-edns_rr_create(allocator_type* allocator)
+edns_rr_create()
 {
     edns_rr_type* err = NULL;
-    if (!allocator) {
-        return NULL;
-    }
-    err = (edns_rr_type*) allocator_alloc(allocator, sizeof(edns_rr_type));
-    if (!err) {
-        ods_log_error("[%s] unable to create edns rr: allocator_alloc() "
-            "failed", edns_str);
-        return NULL;
-    }
-    err->allocator = allocator;
+    CHECKALLOC(err = (edns_rr_type*) malloc(sizeof(edns_rr_type))) ;
     edns_rr_reset(err);
     return err;
 }
@@ -88,7 +79,6 @@ edns_init(edns_data_type* data, uint16_t max_length)
     data->error[3] = (max_length & 0xff00) >> 8;       /* size_hi */
     data->error[4] = max_length & 0x00ff;              /* size_lo */
     data->error[5] = 1; /* Extended RCODE=BAD VERS */
-    return;
 }
 
 
@@ -106,7 +96,6 @@ edns_rr_reset(edns_rr_type* err)
     err->position = 0;
     err->maxlen = 0;
     err->dnssec_ok = 0;
-    return;
 }
 
 
@@ -176,3 +165,11 @@ edns_rr_reserved_space(edns_rr_type* err)
     return err->status == EDNS_NOT_PRESENT ? 0 : (OPT_LEN + OPT_RDATA);
 }
 
+
+void
+edns_rr_cleanup(edns_rr_type* err)
+{
+    if (!err)
+	return;
+    free(err);
+}
